/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "common.h"

/*
 * --------------------------------------------------------------
 * Error functions
 * --------------------------------------------------------------
 */
void Output(Sys* pSys,PRINT ePrint,int iErr,const char* pcFmt,va_list ap)
{
    char* pcErr;
    const char* pcPrefix;
    bool yNL;

    switch(ePrint) {
        default: /* FallThrough */
        case PRINT_INFO:        pcPrefix = "";          yNL = true; break;
        case PRINT_INFO_NONL:   pcPrefix = "";          yNL = false; break;
        case PRINT_WARN:        pcPrefix = "WARNING: "; yNL = true; break;
        case PRINT_DIE:         pcPrefix = "DIE: ";     yNL = true; break;
    }
#ifdef __linux__
    if(IsSyslogEnabled(pSys)) {
        if(iErr) {
            pcErr = strerror(iErr);
            syslog(LOG_USER,"%s%s (%d): ",pcPrefix,pcErr,iErr);
        }
        vsyslog(LOG_USER,pcFmt,ap);
    } else
#endif
    {
        fprintf(stderr,"%s",pcPrefix);
        if(iErr) {
            pcErr = strerror(iErr);
            fprintf(stderr,"Error: %s (%d): ",pcErr,iErr);
        }
        vfprintf(stderr,pcFmt,ap);
        if(yNL) {
            fprintf(stderr,"\n");
        }
        fflush(stderr);
    }
}
