/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef PROTOCOL_H__
#define PROTOCOL_H__

#define MAX_PEB_SIZE_BYTES   (128*1024)

struct CommandArea;
typedef struct CommandArea CommandArea;

struct ResponseArea;
typedef struct ResponseArea ResponseArea;

struct MessageArea;
typedef struct MessageArea MessageArea;

struct AuxArea;
typedef struct AuxArea AuxArea;

typedef enum ERR {
    ERR_NONE = 0,
    ERR_SEQ,
    ERR_COMMAND,
    ERR_BOARD_FLASH,
    ERR_PEB_NUMBER,
    ERR_SUBSECTOR_NUMBER,
    ERR_ERASE,
    ERR_READ,
    ERR_WRITE,
    ERR_FLASH_UNVERIFIED,
    ERR_PEB_SIZE,
    ERR_SUBSECTOR_SIZE,
    ERR_IMAGE_SIZE,
    ERR_CHECKSUM,
    ERR_MULTIBOOT_BYTES,
    ERR_IMAGE_ALIGNMENT,
    ERR_BSP,
    ERR_TIMEOUT,
    ERR_INTERNAL = 0x7fffffff
} ERR;
extern const char* GetErrString(ERR eErr);

typedef enum COMMAND {
    COMMAND_INVALID = 0,
    COMMAND_VERIFY_FLASH,
    COMMAND_VERIFY_IMAGE,
    COMMAND_GET_MULTIBOOT_BYTES,
    COMMAND_ERASE_PEBS,
    COMMAND_READ_PEB,
    COMMAND_WRITE_PEB,
    COMMAND_ERASE_SUBSECTOR,
    COMMAND_READ_SUBSECTOR,
    COMMAND_WRITE_SUBSECTOR,
    COMMAND_SIM_STOP    /* Simulation only */
} COMMAND;
extern const char* GetCommandString(COMMAND eCommand);

typedef enum CONNECTION_MODE {
    CONNECTION_MODE_LOWER,
    CONNECTION_MODE_DUAL_PARALLEL
} CONNECTION_MODE;

#define COMMAND_MAX_ARGS    ((512-(4*2))/4)
struct CommandArea {
    volatile uint32_t qCommand;
    volatile uint32_t pqArgs[COMMAND_MAX_ARGS];
    volatile uint32_t qChecksum;
};

#define FEEDC0DE       0xFEEDC0DE
#define RESPONSE_MAX_DATA   ((512-(4*4))/4)
struct ResponseArea {
    volatile uint32_t qErrCode;
    volatile uint32_t qErrBSP;
    volatile uint32_t pqData[RESPONSE_MAX_DATA];
    volatile uint32_t qChecksum;
    uint32_t qFEEDC0DE;
};

struct MessageArea {
    CommandArea tCommandArea;
    ResponseArea tResponseArea;
    uint32_t pqReserved[(3*1024)/4];
};

struct AuxArea {
    uint8_t pbData[MAX_PEB_SIZE_BYTES];
};

#endif  /* PROTOCOL_H__ */
