/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "common.h"
#include "r5_multiboot.h"
#include "xpseudo_asm_gcc.h"

void r5InitMultibootState(MultibootState* pMS)
{
    /*
     * Inherits clocks/resets from PLM initialization
     */

    /*
     * DIRM_3: R5_UP is only output, assert its mask and value
     */
    *(volatile uint32_t*)0xFF0B0018 = (~EMIO_R5_UP << 16) | ~EMIO_R5_UP;

    /*
     * DIRM_3: R5_UP is output
     */
    *(volatile uint32_t*)0xFF0B02C4 = EMIO_R5_UP;

    /*
     * OEN_3: R5_UP is enabled
     */
    *(volatile uint32_t*)0xFF0B02C8 = EMIO_R5_UP;

    /*
     * Store next multiboot value
     */
    pMS->qMultibootReg = *(volatile uint32_t*)0xF1110004 + 1;

    pMS->qPcieGoodCounter = EMIO_PCIE_GOOD_COUNTER_TIMEOUT;
    pMS->ePcieState = PCIE_STATE_UNKNOWN;
}

void r5SenseMultibootInputs(MultibootState* pMS)
{
    /*
     * Both MULTIBOOT and CPLD_UP have to be asserted
     */
    pMS->yMultibootRequest = (0x0 == ((*(volatile uint32_t*)0xFF0B006C) & (EMIO_MULTIBOOT | EMIO_CPLD_UP)));
    if((*(volatile uint32_t*)0xFF0B006C) & EMIO_PCIE_GOOD) {
        pMS->ePcieState = PCIE_STATE_UP;
        /*
         * Allows for flicker.
         * The bit will toggle once as it trains
         * The r5_app may or may not catch it
         */
        if(pMS->qPcieGoodCounter > 0) {
            pMS->qPcieGoodCounter--;
        }
    } else if(pMS->qPcieGoodCounter > 0) {
        pMS->qPcieGoodCounter--;
        pMS->ePcieState = PCIE_STATE_UNKNOWN;
    } else {
        pMS->ePcieState = PCIE_STATE_DOWN;
    }
}

void r5PerformMultiboot(MultibootState* pMS)
{
    TRACE();
    /*
     * Store scaled multiboot value into PMC_MULTI_BOOT
     */
    *(volatile uint32_t*)0xF1110004 = pMS->qMultibootReg;

    /*
     * Set boot mode user to qspi32
     */
    *(volatile uint32_t*)0xF1260200 = (2 << 12) | (1 << 8);

    if(0x20 == (*(volatile uint32_t*)0xF11A0004 & 0xff)) {
        TRACE();

        /*
         * Production PMC
         * Probably not needed
         */

        /*
         * Set SYSMON_REF_CTRL::SrcSel to PMC_IRO_CLK /2
         */
        *(volatile uint32_t*)0xF1260138 = 0;

        /*
         * Set RST_NONPS, all resets
         */
        *(volatile uint32_t*)0xF1260320 = 0x77;
    } else {
        TRACE();

        /*
         * ES PMC
         */

        /*
         * Set SYSMON_REF_CTRL::SrcSel to PMC_IRO_CLK /2
         */
        *(volatile uint32_t*)0xF1260138 = 0;

        /*
         * Set RST_NONPS, all resets
         */
        *(volatile uint32_t*)0xF1260320 = 0x77;
    }
    TRACE();

    /*
     * Ensure all activity has finished before performing SRST
     */
    dsb();
    isb();

    /*
     * Perform SRST
     *
     * Write to RST_PS[PMC_SRST]
     */
    *(volatile uint32_t*)0xF126031C = 1 << 3;

    __builtin_unreachable();

    /*
     * Not reached
     */
    for(;;) {
        TRACE();
    }
}
