/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include "xparameters.h"
#include "xuartpsv.h"
#include "common.h"
#include "r5_multiboot.h"
/* ------------------------------------------------------------------ */
#ifdef DO_TRACE
void InitUart(void)
{
    XUartPsv tUartPsv;
    XUartPsv_Config* Config;
    int Status;
    
    Config = XUartPsv_LookupConfig(0);
    if( NULL == Config) {
        return;
    }
    Status = XUartPsv_CfgInitialize(&tUartPsv, Config, Config->BaseAddress);
    if (Status != XST_SUCCESS) {
        return;
    }
}
#endif

/* ------------------------------------------------------------------ */

bool IsVerboseEnabled(Sys* pSys)
{
    return false;
}

bool IsSyslogEnabled(Sys* pSys)
{
    return 0;
}

/* ------------------------------------------------------------------ */
int main(void)
{
    MultibootState tMultibootState;
    MultibootState* pMS = &tMultibootState;

#ifdef DO_TRACE
    InitUart();
    TRACE_PRINT("Build info: %s %s\n\r",__DATE__,__TIME__);
#endif
    TRACE();
    r5InitMultibootState(&tMultibootState);
    TRACE();

    for(;;) {
        r5SenseMultibootInputs(pMS);

        if(pMS->yMultibootRequest || (PCIE_STATE_DOWN == pMS->ePcieState)) {
            TRACE();
            r5PerformMultiboot(pMS);
            __builtin_unreachable();
            for(;;) {
                ;
            }
        }
        TRACE();
    }
    return 0;
}
