#! /usr/bin/env bash
#
# Copyright 2021 Xilinx, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
set -x
set -e
source locate.sh
L_CURLY='{'
R_CURLY='}'
BIF="my.bif"

rm -f ${BIF}

source ${MY_DIR}/decode_design_dir.sh $1
BIN="${DESIGN}"
cat << EOF1 >>${BIF}
new_bif:
${L_CURLY}
    image
    ${L_CURLY}
        ${L_CURLY}
            type = bootimage
            file = ${DESIGN_PDI}
        ${R_CURLY}
    ${R_CURLY}
EOF1

source ${MY_DIR}/decode_app_dir.sh $2
BIN="${BIN}-${APP_NAME}-${APP_R5_CORE}"
BIF1="${BIF}1"
rm -f "${BIF1}"
cat << EOF2 >${BIF1}
    image
    ${L_CURLY}
        name = lpd
        partition
        ${L_CURLY}
            core = ${APP_R5_CORE}
            file = ${APP_R5_ELF}
        ${R_CURLY}
    ${R_CURLY}
EOF2
ELF1="${APP_R5_ELF}"
CORE1="${APP_R5_CORE}"
APP_NAME1="${APP_NAME}"

if [[ "X$3" != "X" ]] ; then
    source ${MY_DIR}/decode_app_dir.sh $3
    BIN="${BIN}-${APP_NAME}-${APP_R5_CORE}"
    BIF2="${BIF}2"
    rm -f "${BIF2}"
    cat << EOF3 >${BIF2}
    image
    ${L_CURLY}
        name = lpd
        partition
        ${L_CURLY}
            core = ${APP_R5_CORE}
            file = ${APP_R5_ELF}
        ${R_CURLY}
    ${R_CURLY}
EOF3
    ELF2="${APP_R5_ELF}"
    CORE2="${APP_R5_CORE}"
    APP_NAME2="${APP_NAME}"
else
    ELF2=""
    CORE2=""
    APP_NAME2=""
fi

if [[ "X${CORE2}" != "X" ]]; then
    if [[ "X${CORE1}" = "X${CORE2}" ]]; then
        echo "Error: Both elf files use the same core ${CORE1}"
        exit 1
    fi
    if [[ ${CORE1} = "r5-0" ]]; then
        cat ${BIF1} >> ${BIF}
        cat ${BIF2} >> ${BIF}
    else
        cat ${BIF2} >> ${BIF}
        cat ${BIF1} >> ${BIF}
    fi
    rm -f ${BIF1}
    rm -f ${BIF2}
else
    cat ${BIF1} >> ${BIF}
    rm -f ${BIF1}
fi

BUILD="bootgen-${BIN}"
BIN="${BIN}.bin"
cat << EOF4 >>${BIF}
${R_CURLY}
EOF4

rm -rf ${BUILD}
mkdir -p ${BUILD}
mv ${BIF} ${BUILD}
pushd ${BUILD}
    cp ${ELF1} "${APP_NAME1}-${CORE1}.elf"
    cp ${DESIGN_PDI} .
    if [[ "X${ELF2}" != "X" ]]; then
        cp ${ELF2} "${APP_NAME2}-${CORE2}.elf"
    fi
    bootgen \
            -arch versal \
            -fill 0xff \
            -image ${BIF} \
            -log trace \
            -w on \
            -o "${BIN}"
popd

exit 0
