/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef R5_COMMS_H__
#define R5_COMMS_H__

#define TIMEOUT_FEEDC0DE            1000
#define TIMEOUT_VERIFY_FLASH        1000
#define TIMEOUT_VERIFY_IMAGE        10000
#define TIMEOUT_GET_MULTIBOOT_BYTES 1000
#define TIMEOUT_ERASE_PEB           100
#define TIMEOUT_READ_PEB            1000
#define TIMEOUT_WRITE_PEB           1000
#define TIMEOUT_ERASE_SUBSECTOR     3000
#define TIMEOUT_READ_SUBSECTOR      1000
#define TIMEOUT_WRITE_SUBSECTOR     3000
#define TIMEOUT_SIM_STOP            1000

extern bool     WaitForSyncWord(Sys* pSys,uint32_t qTimeoutMs);
extern uint32_t VerifyFlash(
        Sys* pSys,
        uint32_t qConnectionMode,
        uint32_t qBusWidth,
        uint32_t qQspiPeripheralInputSpeedHz,
        uint32_t qPrescaler,
        uint32_t qCacheCoherentDMA,
        uint32_t qPEBSize,
        uint32_t qNumPEBs,
        uint32_t qSubsectorSize,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t VerifyImage(
        Sys* pSys,
        uint32_t qImageOffset,
        uint32_t qImageSize,
        uint32_t qImageChecksum,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t GetMultibootBytes(
        Sys* pSys,
        OUTPUT_PARAMETER uint32_t* pqErrBSP,
        OUTPUT_PARAMETER uint32_t* pqMultibootBytes
        );
extern uint32_t ErasePEBs(
        Sys* pSys,
        uint32_t qPEBStart,
        uint32_t qPEBEnd,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t ReadPEBInternal(
        Sys* pSys,
        uint32_t qPEBNumber,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t ReadPEB(
        Sys* pSys,
        uint32_t qPEBNumber,
        OUTPUT_PARAMETER uint32_t* pqErrBSP,
        OUTPUT_PARAMETER uint8_t* pbDst
        );
extern uint32_t WritePEBInternal(
        Sys* pSys,
        uint32_t qPEBNumber,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t WritePEB(
        Sys* pSys,
        uint32_t qPEBNumber,
        const uint8_t* pbSrc,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t EraseSubsector(
        Sys* pSys,
        uint32_t qSubsectorNumber,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t ReadSubsectorInternal(
        Sys* pSys,
        uint32_t qSubsectorNumber,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t ReadSubsector(
        Sys* pSys,
        uint32_t qSubsectorNumber,
        OUTPUT_PARAMETER uint32_t* pqErrBSP,
        OUTPUT_PARAMETER uint8_t* pbDst
        );
extern uint32_t WriteSubsectorInternal(
        Sys* pSys,
        uint32_t qSubsectorNumber,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t WriteSubsector(
        Sys* pSys,
        uint32_t qSubsectorNumber,
        const uint8_t* pbSrc,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );
extern uint32_t SimStop(
        Sys* pSys,
        OUTPUT_PARAMETER uint32_t* pqErrBSP
        );

#endif  /* R5_COMMS_H__ */
