/*
 * Copyright 2021 Xilinx, Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <x86_app.h>

static void SimPause(ProgEnvFuncs* pPEF)
{
    int iErr;

    iErr = sched_yield();
    if(iErr) {
        Die(pPEF->pSys,iErr,"Pause()");
        /* Not reached */
    }
}

static void SimDataBarrier(ProgEnvFuncs* pPEF,CACHE_REGION eCacheRegion)
{
    ;
}

static uint32_t SimVerifyFlash(
    ProgEnvFuncs* pPEF,
    OUTPUT_PARAMETER volatile uint32_t* pqErrBSP
    )
{
    uint32_t qRet;

    if(1
        && (pPEF->tLS.tVerifyFlashArgsIn.qConnectionMode == pPEF->pSys->pBI->qConnectionMode)
        && (pPEF->tLS.tVerifyFlashArgsIn.qBusWidth == pPEF->pSys->pBI->qBusWidth)
        && (pPEF->tLS.tVerifyFlashArgsIn.qQspiPeripheralInputSpeedHz == pPEF->pSys->pBI->qQspiPeripheralInputSpeedHz)
        && (pPEF->tLS.tVerifyFlashArgsIn.qPrescaler == pPEF->pSys->pBI->qPrescaler)
        && (pPEF->tLS.tVerifyFlashArgsIn.qCacheCoherentDMA == pPEF->pSys->pBI->qCacheCoherentDMA)
        && (pPEF->tLS.tVerifyFlashArgsIn.qPEBSize == pPEF->pSys->pBI->qPEBSize)
        && (pPEF->tLS.tVerifyFlashArgsIn.qNumPEBs == pPEF->pSys->pBI->qNumPEBs)
        && (pPEF->tLS.tVerifyFlashArgsIn.qSubsectorSize == pPEF->pSys->pBI->qSubsectorSize)
    ) {
        *pqErrBSP = 0;
        qRet = ERR_NONE;
    } else {
        *pqErrBSP = ~0;
        qRet = ERR_BOARD_FLASH;
    }
    return qRet;
}

static uint32_t SimErasePEBs(
    ProgEnvFuncs* pPEF,
    uint32_t qPEBStart,
    uint32_t qPEBEnd,
    OUTPUT_PARAMETER volatile uint32_t* pqErrBSP
    )
{
    uint8_t* pbDst;
    size_t count;
    uint32_t qRet;
    SimData* pSimData;

    *pqErrBSP = 0;
    pSimData = (SimData*)pPEF->pSys->pDevData;
    if(MAP_FAILED != pSimData->pxFlashMap) {
        pbDst = (uint8_t*)(pSimData->pxFlashMap) + (qPEBStart * pPEF->tLS.tVerifiedFlashArgs.qPEBSize);
        count = pPEF->tLS.tVerifiedFlashArgs.qPEBSize * (qPEBEnd - qPEBStart + 1);
        (void*)memset((void*)pbDst,0xff,count);
    }

    qRet = ERR_NONE;
    return (uint32_t)qRet;
}

static uint32_t SimReadPEB(
    ProgEnvFuncs* pPEF,
    uint32_t qPEBNumber,
    OUTPUT_PARAMETER volatile uint32_t* pqErrBSP,
    OUTPUT_PARAMETER uint8_t* pbDst
    )
{
    uint8_t* pbSrc;
    uint32_t qRet;
    SimData* pSimData;

    *pqErrBSP = 0;
    pSimData = (SimData*)pPEF->pSys->pDevData;
    if(MAP_FAILED != pSimData->pxFlashMap) {
        pbSrc = (uint8_t*)(pSimData->pxFlashMap) + (qPEBNumber * pPEF->tLS.tVerifiedFlashArgs.qPEBSize);
        (void)memcpy((void*)pbDst,pbSrc,pPEF->tLS.tVerifiedFlashArgs.qPEBSize);
    }

    qRet = ERR_NONE;
    return (uint32_t)qRet;
}

static uint32_t SimWritePEB(
    ProgEnvFuncs* pPEF,
    uint32_t qPEBNumber,
    const uint8_t* pbSrc,
    OUTPUT_PARAMETER volatile uint32_t* pqErrBSP
    )
{
    uint8_t* pbDst;
    uint32_t qRet;
    SimData* pSimData;

    *pqErrBSP = 0;
    pSimData = (SimData*)pPEF->pSys->pDevData;
    if(MAP_FAILED != pSimData->pxFlashMap) {
        pbDst = (uint8_t*)(pSimData->pxFlashMap) + (qPEBNumber * pPEF->tLS.tVerifiedFlashArgs.qPEBSize);
        (void)memcpy((void*)pbDst,(void*)pbSrc,pPEF->tLS.tVerifiedFlashArgs.qPEBSize);
    }

    qRet = ERR_NONE;
    return (uint32_t)qRet;
}

static uint32_t SimGetMultibootBytes(
    ProgEnvFuncs* pPEF,
    OUTPUT_PARAMETER volatile uint32_t* pqErrBSP,
    OUTPUT_PARAMETER uint32_t* pqMultibootBytes
    )
{
    *pqErrBSP = 0;
    switch((char)pPEF->pSys->qMultibootChar) {
    default: /* FallThrough */
    case 'a':
        *pqMultibootBytes = 0;
        break;

    case 'b':
        *pqMultibootBytes = pPEF->pSys->pBI->qPEBOffsetB * pPEF->tLS.tVerifiedFlashArgs.qPEBSize;
        break;

    case 'g':
        *pqMultibootBytes = pPEF->pSys->pBI->qPEBOffsetG * pPEF->tLS.tVerifiedFlashArgs.qPEBSize;
        break;
    }

    return (uint32_t)ERR_NONE;
}

static bool MapSimFlash(Sys* pSys)
{
    int iFile;
    int iFlags;
    int iMode;
    struct stat tStat;
    uint32_t qNumBytes;
    SimData* pSimData;

    qNumBytes = pSys->pBI->qPEBSize * pSys->pBI->qNumPEBs;
    pSimData = (SimData*)pSys->pDevData;

    if(0 == strcmp(pSys->pcSimFlash,"-")) {
        pSimData->pxFlashMap = MAP_FAILED;
        return true;
    }

    iFlags = O_RDWR;
    iMode = 0
        | S_IRUSR
        | S_IWUSR
        | S_IRGRP
        | S_IROTH
        ;
    if(-1 == stat(pSys->pcSimFlash,&tStat)) {
        iFlags |= O_CREAT;
    } else if(!S_ISREG(tStat.st_mode)) {
        Die(pSys,errno,"SimFlash file %s is not a regular file",pSys->pcSimFlash);
        /* Not reached */
        return false;
    } else if(tStat.st_size != qNumBytes) {
        Die(pSys,errno,"SimFlash file %s size is %u, should be %u",
                pSys->pcSimFlash,
                (uint32_t)tStat.st_size,
                qNumBytes
                );
        /* Not reached */
        return false;
    }

    iFile = open(pSys->pcSimFlash,iFlags,iMode);
    if(iFile < 0) {
        Die(pSys,errno,"open('%s')",pSys->pcSimFlash);
        /* Not reached */
        return false;
    }

    if(iFlags & O_CREAT) {
        if(0 != ftruncate(iFile,qNumBytes)) {
            Die(pSys,errno,"ftruncate('%s')",pSys->pcSimFlash);
            /* Not reached */
            return false;
        }
    }

    pSimData->pxFlashMap = mmap(
                            0,                          /* addr */
                            qNumBytes,                  /* length */
                            PROT_READ | PROT_WRITE,     /* prot */
                            MAP_SHARED,                 /* flags */
                            iFile,                      /* fd */
                            0                           /* offset */
                            );
    if(MAP_FAILED == pSimData->pxFlashMap) {
        Die(pSys,errno,"mmap('%s')",pSys->pcSimFlash);
        /* Not reached */
        return false;
    }

    if(iFlags & O_CREAT) {
        memset(pSimData->pxFlashMap,0xff,qNumBytes);
    }

    if(0 != close(iFile)) {
        Die(pSys,errno,"close('%s')",pSys->pcSimFlash);
        /* Not reached */
        return false;
    }

    return true;
}

static bool UnmapSimFlash(Sys* pSys)
{
    bool yRet = true;
    uint32_t qNumBytes;
    int iErr;
    SimData* pSimData;

    pSimData = (SimData*)pSys->pDevData;
    if(MAP_FAILED != pSimData->pxFlashMap) {
        qNumBytes = pSys->pBI->qPEBSize * pSys->pBI->qNumPEBs;
        iErr = munmap(pSimData->pxFlashMap,qNumBytes);
        if(iErr) {
            Warn(pSys,errno,"munmap('%s')",pSys->pcSimFlash);
            yRet = false;
        }
        pSimData->pxFlashMap = MAP_FAILED;
    }
    return yRet;
}

void* SimulateR5(void* pxArg)
{
    ProgEnvFuncs* pPEF = (ProgEnvFuncs*)pxArg;

    r5InitPEF(pPEF);
    while(r5StepPEF(pPEF)) {
        SimPause(pPEF);
    }

    return 0;
}

bool OpenSim(Sys* pSys)
{
    ProgEnvFuncs* pPEF;
    int iRet;
    SimData* pSimData;

    pSys->pDevData = calloc(1,sizeof(SimData));
    if(!pSys->pDevData) {
        Die(pSys,errno,"%s",__func__);
        /* Not reached */
        return false;
    }

    if(!MapSimFlash(pSys)) {
        /* Not reached */
        return false;
    }

    pSimData = (SimData*)pSys->pDevData;
    pPEF = &pSimData->tProgEnvFuncs;

    pPEF->pSys = pSys;
    pSys->pMA = pPEF->pMA = &pSimData->tMA;
    pSys->pAA = pPEF->pAA = &pSimData->tAA;
    pPEF->pPA = &pSimData->tPA;

    pPEF->rDataBarrier = SimDataBarrier;
    pPEF->rVerifyFlash = SimVerifyFlash;
    pPEF->rErasePEBs = SimErasePEBs;
    pPEF->rReadPEB = SimReadPEB;
    pPEF->rWritePEB = SimWritePEB;
    pPEF->rGetMultibootBytes = SimGetMultibootBytes;

    iRet = pthread_create(&pSimData->oThread,NULL,SimulateR5,pPEF);
    if(0 != iRet) {
        UnmapSimFlash(pSys);
        free(pSys->pDevData);
        pSys->pDevData = 0;
        Die(pSys,iRet,"SimulateR5()");
        /* Not reached */
        return false;
    }

    return true;
}

bool CloseSim(Sys* pSys)
{
    bool yRet = true;
    int iRet;
    uint32_t qErrBSP;
    SimData* pSimData = (SimData*)(pSys->pDevData);

    (void)SimStop(pSys,&qErrBSP);
    iRet = pthread_join(pSimData->oThread,0);
    if(0 != iRet) {
        Warn(pSys,iRet,"SimulateR5()");
        yRet = false;
    }
    yRet &= UnmapSimFlash(pSys);
    free(pSys->pDevData);
    pSys->pDevData = 0;
    return yRet;
}
